﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using DumpTruck.Entities;
using DumpTruck.MovementStrategy;


namespace DumpTruck.DrawningObjects
{
    public class DrawningCar
    {
        public EntityCar? EntityCar { get; protected set; }

        private int _pictureWidth;

        private int _pictureHeight;

        protected int _startPosX;

        protected int _startPosY;

        private readonly int _carWidth = 110;

        private readonly int _carHeight = 60;

        /// <summary>
        /// Координата X объекта
        /// </summary>
        public int GetPosX => _startPosX;
        /// <summary>
        /// Координата Y объекта
        /// </summary>
        public int GetPosY => _startPosY;
        /// <summary>
        /// Ширина объекта
        /// </summary>
        public int GetWidth => _carWidth;
        /// <summary>
        /// Высота объекта
        /// </summary>
        public int GetHeight => _carHeight;

        /// <summary>
        /// Получение объекта IMoveableObject из объекта DrawningCar
        /// </summary>
        public IMoveableObject GetMoveableObject => new DrawningObjectCar(this);

        public DrawningCar(int speed, double weight, Color bodyColor, int width, int height)
        {
            if (width < _carWidth || height < _carHeight)
            {
                return;
            }
            _pictureWidth = width;
            _pictureHeight = height;

            EntityCar = new EntityCar(speed, weight, bodyColor);



        }

        protected DrawningCar(int speed, double weight, Color bodyColor, int
width, int height, int carWidth, int carHeight)
        {
            if (width <= _carWidth || height <= _carHeight)
            {
                return;
            }
            _pictureWidth = width;
            _pictureHeight = height;
            _carWidth = carWidth;
            _carHeight = carHeight;
            EntityCar = new EntityCar(speed, weight, bodyColor);
        }

        public void SetPosition(int x, int y)
        {
            if (x < 0 || x >= _pictureWidth || y < 0 || y >= _pictureHeight)
            {
                _startPosX = 0;
                _startPosY = 0;
            }
            _startPosX = x;
            _startPosY = y;
        }

        public void MoveTransport(DirectionType direction)
        {
            if (!CanMove(direction) || EntityCar == null)
            {
                return;
            }
            switch (direction)
            {
                //влево
                case DirectionType.Left:
                    _startPosX -= (int)EntityCar.Step;
                    break;
                //вверх
                case DirectionType.Up:
                    _startPosY -= (int)EntityCar.Step;
                    break;
                // вправо
                case DirectionType.Right:
                    _startPosX += (int)EntityCar.Step;
                    break;
                //вниз
                case DirectionType.Down:
                    _startPosY += (int)EntityCar.Step;
                    break;
            }
        }

        public bool CanMove(DirectionType direction)
        {
            if (EntityCar == null)
            {
                return false;
            }
            return direction switch
            {
                //влево
                DirectionType.Left => _startPosX - EntityCar.Step > 0,
                //вверх
                DirectionType.Up => _startPosY - EntityCar.Step > 0,
                // вправо
                DirectionType.Right => _startPosX + EntityCar.Step + _carWidth < _pictureWidth,
                //вниз
                DirectionType.Down => _startPosY + EntityCar.Step + _carHeight < _pictureHeight,
                _ => false,
            };
        }


        public virtual void DrawTransport(Graphics g)
        {
            if (EntityCar == null)
            {
                return;
            }

            Pen pen = new Pen(Color.Black);
            Brush brush = new SolidBrush(EntityCar.BodyColor);

            //границы автомобиля
            g.FillRectangle(brush, _startPosX, _startPosY + 35, 110, 10);
            g.FillRectangle(brush, _startPosX + 85, _startPosY, 25, 35);
            g.FillEllipse(brush, _startPosX, _startPosY + 35 + 10, 15, 15);
            g.FillEllipse(brush, _startPosX + 15, _startPosY + 35 + 10, 15, 15);
            g.FillEllipse(brush, _startPosX + 95, _startPosY + 35 + 10, 15, 15);

        }

        public void setColor(Color color)
        {
            EntityCar.setColor(color);
        }
    }
}
